@@ -1,4 +1,4 @@
-plot.MIPCA <- function(x,choice="all",axes=c(1,2),new.plot=TRUE,main=NULL,level.conf=0.95, ...){
+plot.MIPCA <- function(x,choice="all",axes=c(1,2),new.plot=TRUE,main=NULL,level.conf=0.95, graph.type=c("ggplot","classic"), ...){
 
 ####
  procrustes <- function(amat, target, orthogonal = FALSE, translate = FALSE,
@@ -94,15 +94,20 @@ for (i in 1:length(res$res.MI)){
 
 ####
 if (!is.null(main)) title <- main
+graph.type <- match.arg(graph.type[1],c("ggplot","classic"))
+if (graph.type=="ggplot") graph <- list()
 if ((choice=="all")|(choice=="ind.proc")){
-  if (new.plot) dev.new()
+  if ((new.plot)&!nzchar(Sys.getenv("RSTUDIO_USER_IDENTITY"))) dev.new()
   oo=FactoMineR::PCA(res.procrustes,ind.sup=c((nrow(res$call$X)+1):nrow(res.procrustes)),scale.unit=FALSE,graph=FALSE)
   oo$eig=reference$eig
   if (is.null(rownames(res$call$X))) rownames(res$call$X) <- 1:nrow(res$call$X)
   el=coord.ellipse(cbind.data.frame(as.factor(rep(rownames(res$call$X),res$call$nboot)),oo$ind.sup$coord[,axes]),level.conf=level.conf) 
   if (is.null(main)) title="Multiple imputation using Procrustes" 
-  plot(oo,axes=axes,col.ind.sup=rep(1:nrow(res$call$X),res$call$nboot),label="ind",ellipse=el,col.quali="black", title=title,invisible="ind.sup",new.plot=FALSE,graph.type="classic")
-
+  PlotIndProc <- plot(oo,axes=axes,col.ind.sup=rep(1:nrow(res$call$X),res$call$nboot),label="ind",ellipse=el,col.quali=1, title=title,invisible="ind.sup",new.plot=FALSE,graph.type=graph.type)
+  if (graph.type=="ggplot"){
+    print(PlotIndProc)
+    graph$PlotIndProc <- PlotIndProc
+  }
 #  if (!is.null(add.tab)){
 #    vrai = PCA(add.tab,graph=FALSE,scale=res$call$scale)
 #    tourne <- procrustes(vrai$ind$coord[,axes], reference$ind$coord[,axes],orthogonal = TRUE, translate = TRUE, magnify = TRUE)$rmat
@@ -116,7 +121,11 @@ if ((choice=="all")|(choice=="dim")){
   ooo=FactoMineR::PCA(res.dim,quanti.sup=(ncol(res$call$X)+1):ncol(res.dim),scale.unit=res$call$scale,graph=FALSE)
   ooo$eig=reference$eig
   if (is.null(main)) title="Projection of the Principal Components"  
-  plot(ooo,choi="var",axes=axes,title=title,label="none",new.plot=FALSE,invisible="var",graph.type="classic")
+  PlotDim <- plot(ooo,choi="var",axes=axes,title=title,label="none",new.plot=FALSE,invisible="var",graph.type=graph.type)
+  if (graph.type=="ggplot"){
+    print(PlotDim)
+    graph$PlotDim <- PlotDim
+  }
 }
 
 if ((choice=="all")|(choice=="ind.supp")){
@@ -124,8 +133,12 @@ if ((choice=="all")|(choice=="ind.supp")){
   oo=FactoMineR::PCA(res.supp,ind.sup=c((nrow(res$call$X)+1):nrow(res.supp)),scale.unit=res$call$scale,graph=FALSE,ncp=ncp)
   el=coord.ellipse(cbind.data.frame(as.factor(rep(rownames(res$call$X),res$call$nboot)),oo$ind.sup$coord),level.conf = level.conf,axes = axes)
   if (is.null(main)) title="Supplementary projection"    
-  plot(oo,axes=axes,col.ind.sup=rep(1:nrow(res$call$X),res$call$nboot),label="ind",ellipse=el,col.quali="black",
-    title=title,invisible="ind.sup",new.plot=FALSE,graph.type="classic")
+  PlotIndSupp <- plot(oo,axes=axes,col.ind.sup=rep(1:nrow(res$call$X),res$call$nboot),label="ind",ellipse=el,col.quali=1,
+    title=title,invisible="ind.sup",new.plot=FALSE, graph.type=graph.type)
+  if (graph.type=="ggplot"){
+    print(PlotIndSupp)
+    graph$PlotIndSupp <- PlotIndSupp
+  }
 #  if (!is.null(add.tab)){
 #    dele = PCA(rbind.data.frame(rec.pca,add.tab),ind.sup=c((nrow(res$call$X)+1):(2*nrow(res$call$X))),scale=scale,graph=FALSE)
 #    points(dele$ind.sup$coord[,axes],col=2)
@@ -147,7 +160,16 @@ if ((choice=="all")|(choice=="var")){
   colnames(res.var)[1:ncol(res$call$X)]=colnames(res$call$X)
   oo=FactoMineR::PCA(res.var,quanti.sup=c((ncol(res$call$X)+1):ncol(res.var)),scale.unit=res$call$scale,graph=FALSE)
   if (is.null(main)) title="Variable representation"    
-  plot(oo, axes=axes, choix = "var", title=title,invisible = "quanti.sup", col.hab = color[1:ncol(res$call$X)],new.plot=FALSE,graph.type="classic")
-  for (k in 1:res$call$nboot) points(oo$quanti.sup$coord[((k-1)*ncol(res$call$X)+1):(k*ncol(res$call$X)),axes[1]], oo$quanti.sup$coord[((k-1)*ncol(res$call$X)+1):(k*ncol(res$call$X)),axes[2]], col = color[1:ncol(res$call$X)], pch = 15, cex = 0.3)
+  PlotVar <- plot(oo, axes=axes, choix = "var", title=title,invisible = "quanti.sup", col.hab = color[1:ncol(res$call$X)],new.plot=FALSE,graph.type=graph.type)
+  if (graph.type=="classic") {
+    for (k in 1:res$call$nboot) points(oo$quanti.sup$coord[((k-1)*ncol(res$call$X)+1):(k*ncol(res$call$X)),axes[1]], oo$quanti.sup$coord[((k-1)*ncol(res$call$X)+1):(k*ncol(res$call$X)),axes[2]], col = color[1:ncol(res$call$X)], pch = 15, cex = 0.3)
+  } else {
+    y <- variables <- NULL ## to avoid no visible binding for global variable
+    dta <- cbind.data.frame(x=oo$quanti.sup$coord[,axes[1]],y=oo$quanti.sup$coord[,axes[2]],variables=rep(colnames(res$call$X),res$call$nboot))
+    PlotVar <- PlotVar + geom_point(data=dta,aes(x=x,y=y,color=variables),alpha=0.5,shape=20,size=2)+guides(color = guide_legend(title=NULL,override.aes = list(alpha=1, size=4)))
+    print(PlotVar)
+    graph$PlotVar <- PlotVar
+  }
 }
+  if (graph.type=="ggplot") return(graph)
 }
